@@ -247,9 +247,18 @@ type curlProxyType =
   | CURLPROXY_SOCKS4A (** added in 7.18.0 *)
   | CURLPROXY_SOCKS5_HOSTNAME (** added in 7.18.0 *)
 
+type data_source =
+  | String of string (** Equivalent to `CURLMIME_DATA` *)
+  | File of string  (** Equivalent to `CURLMIME_FILEDATA` *)
+
 type curlMIMEPartData =
   | CURLMIME_DATA of string
   | CURLMIME_FILEDATA of string
+  | CURLMIME_DATA_WITH_NAME of {
+      data: data_source;
+      name: string option;
+      filename: string option;
+    }
 
 type curlMIMEEncoding =
   | CURLMIME_8BIT
@@ -315,6 +324,23 @@ type curlPostRedir =
 | REDIR_POST_302
 | REDIR_POST_303
 
+type curlSslOption =
+  | CURLSSLOPT_ALLOW_BEAST
+  | CURLSSLOPT_NO_REVOKE
+  | CURLSSLOPT_NO_PARTIALCHAIN
+  | CURLSSLOPT_REVOKE_BEST_EFFORT
+  | CURLSSLOPT_NATIVE_CA
+  | CURLSSLOPT_AUTO_CLIENT_CERT
+
+type curlWritefuncOpt = string -> [`Full | `Partial of int | `Pause]
+
+type 'a xfer_result = Proceed of 'a | Pause | Abort
+
+type write_result = unit xfer_result
+type read_result = string xfer_result
+
+let proceed = Proceed ()
+
 type curlOption =
   | CURLOPT_WRITEFUNCTION of (string -> int)
   | CURLOPT_READFUNCTION of (int -> string)
@@ -412,6 +438,7 @@ type curlOption =
   | CURLOPT_FTPSSL of curlFTPSSL
   | CURLOPT_POSTFIELDSIZELARGE of int64
   | CURLOPT_TCPNODELAY of bool
+  | CURLOPT_TCP_FASTOPEN of bool
   | CURLOPT_FTPSSLAUTH of curlFTPSSLAuth
   | CURLOPT_IOCTLFUNCTION of (t -> curlIOCmd -> curlIOErr)
   | CURLOPT_FTPACCOUNT of string
@@ -443,6 +470,7 @@ type curlOption =
   | CURLOPT_SEEKFUNCTION of (int64 -> curlSeek -> curlSeekResult)
   | CURLOPT_AUTOREFERER of bool
   | CURLOPT_OPENSOCKETFUNCTION of (Unix.file_descr -> unit)
+(*   | CURLOPT_CLOSESOCKETFUNCTION of (Unix.file_descr -> unit) *)
   | CURLOPT_PROXYTYPE of curlProxyType
   | CURLOPT_PROTOCOLS of curlProto list
   | CURLOPT_REDIR_PROTOCOLS of curlProto list
@@ -460,6 +488,12 @@ type curlOption =
   | CURLOPT_MIMEPOST of curlMIMEPart list
   | CURLOPT_SSHKNOWNHOSTS of string
   | CURLOPT_SSHKEYFUNCTION of (curlKHMatch -> string -> curlKHStat)
+  | CURLOPT_BUFFERSIZE of int
+  | CURLOPT_DOH_URL of string
+  | CURLOPT_SSL_OPTIONS of curlSslOption list
+  | CURLOPT_WRITEFUNCTION2 of (string -> write_result)
+  | CURLOPT_READFUNCTION2 of (int -> read_result)
+  | CURLOPT_XFERINFOFUNCTION of (int64 -> int64 -> int64 -> int64 -> bool)
 
 type initOption =
   | CURLINIT_GLOBALALL
@@ -505,6 +539,8 @@ type curlInfo =
   | CURLINFO_LOCAL_PORT
   | CURLINFO_CONDITION_UNMET
   | CURLINFO_CERTINFO
+  | CURLINFO_ACTIVESOCKET
+  | CURLINFO_HTTP_VERSION
 
 type curlInfoResult =
   | CURLINFO_String of string
@@ -512,11 +548,42 @@ type curlInfoResult =
   | CURLINFO_Double of float
   | CURLINFO_StringList of string list
   | CURLINFO_StringListList of string list list
+  | CURLINFO_Socket of Unix.file_descr
+  | CURLINFO_Version of curlHTTPVersion
+
+type curlSslBackend =
+  | CURLSSLBACKEND_NONE
+  | CURLSSLBACKEND_OPENSSL
+  | CURLSSLBACKEND_GNUTLS
+  | CURLSSLBACKEND_NSS
+  | CURLSSLBACKEND_GSKIT
+  | CURLSSLBACKEND_WOLFSSL
+  | CURLSSLBACKEND_SCHANNEL
+  | CURLSSLBACKEND_SECURETRANSPORT
+  | CURLSSLBACKEND_MBEDTLS
+  | CURLSSLBACKEND_MESALINK
+  | CURLSSLBACKEND_BEARSSL
+
+type curlSslSet =
+  | CURLSSLSET_OK
+  | CURLSSLSET_UNKNOWN_BACKEND
+  | CURLSSLSET_TOO_LATE
+  | CURLSSLSET_NO_BACKENDS
 
 exception NotImplemented of string
 
 let () = Callback.register_exception "Curl.NotImplemented" (NotImplemented "")
 
+exception CurlSslSetException of curlSslSet
+
+let () =
+  Callback.register_exception "CurlSslSetException"
+    (CurlSslSetException CURLSSLSET_OK)
+
+external global_sslset : curlSslBackend -> unit = "caml_curl_global_sslset"
+external global_sslset_str : string -> unit = "caml_curl_global_sslset_str"
+external global_sslsetavail : unit -> curlSslBackend list = "caml_curl_global_sslsetavail"
+external global_sslsetavail_str : unit -> string list = "caml_curl_global_sslsetavail_str"
 external global_init : initOption -> unit = "caml_curl_global_init"
 external global_cleanup : unit -> unit = "caml_curl_global_cleanup"
 external init : unit -> t = "caml_curl_easy_init"
@@ -559,9 +626,15 @@ external pause : t -> pauseOption list -> unit = "caml_curl_pause"
 let set_writefunction conn closure =
   setopt conn (CURLOPT_WRITEFUNCTION closure)
 
+let set_writefunction2 conn closure =
+  setopt conn (CURLOPT_WRITEFUNCTION2 closure)
+
 let set_readfunction conn closure =
   setopt conn (CURLOPT_READFUNCTION closure)
 
+let set_readfunction2 conn closure =
+  setopt conn (CURLOPT_READFUNCTION2 closure)
+
 let set_infilesize conn size =
   setopt conn (CURLOPT_INFILESIZE size)
 
@@ -733,6 +806,9 @@ let set_krb4level conn krb4level =
 let set_progressfunction conn closure =
   setopt conn (CURLOPT_PROGRESSFUNCTION closure)
 
+let set_xferinfofunction conn closure =
+  setopt conn (CURLOPT_XFERINFOFUNCTION closure)
+
 let set_sslverifypeer conn flag =
   setopt conn (CURLOPT_SSLVERIFYPEER flag)
 
@@ -847,6 +923,9 @@ let set_postfieldsizelarge conn size =
 let set_tcpnodelay conn flag =
   setopt conn (CURLOPT_TCPNODELAY flag)
 
+let set_tcpfastopen conn flag =
+  setopt conn (CURLOPT_TCP_FASTOPEN flag)
+
 let set_ftpsslauth conn auth =
   setopt conn (CURLOPT_FTPSSLAUTH auth)
 
@@ -940,6 +1019,11 @@ let set_autoreferer conn b =
 let set_opensocketfunction conn closure =
   setopt conn (CURLOPT_OPENSOCKETFUNCTION closure)
 
+(*
+let set_closesocketfunction conn closure =
+  setopt conn (CURLOPT_CLOSESOCKETFUNCTION closure)
+*)
+
 let set_proxytype conn ptype =
   setopt conn (CURLOPT_PROXYTYPE ptype)
 
@@ -990,6 +1074,15 @@ let set_sshknownhosts conn s =
 let set_sshkeyfunction conn f =
   setopt conn (CURLOPT_SSHKEYFUNCTION f)
 
+let set_buffersize conn buffersize =
+  setopt conn (CURLOPT_BUFFERSIZE buffersize)
+
+let set_doh_url conn url =
+  setopt conn (CURLOPT_DOH_URL url)
+
+let set_ssl_options conn opts =
+  setopt conn (CURLOPT_SSL_OPTIONS opts)
+
 let get_effectiveurl conn =
   match (getinfo conn CURLINFO_EFFECTIVE_URL) with
   | CURLINFO_String s -> s
@@ -1156,6 +1249,11 @@ let get_lastsocket conn =
   | CURLINFO_Long l -> l
   | _ -> 0
 
+let get_activesocket conn =
+  match (getinfo conn CURLINFO_ACTIVESOCKET) with
+  | CURLINFO_Socket s -> Some s
+  | _ -> None
+
 let get_ftpentrypath conn =
   match (getinfo conn CURLINFO_FTP_ENTRY_PATH) with
   | CURLINFO_String s -> s
@@ -1181,6 +1279,11 @@ let get_conditionunmet conn =
   | CURLINFO_Long n -> n <> 0
   | _ -> assert false
 
+let get_http_version conn =
+  match (getinfo conn CURLINFO_HTTP_VERSION) with
+  | CURLINFO_Version v -> v
+  | _ -> assert false
+
 let () =
   Callback.register_exception "CurlException"
     (CurlException (CURLE_OK, 0, ""))
@@ -1192,7 +1295,9 @@ class handle =
     method perform = perform conn
     method cleanup = cleanup conn
     method set_writefunction closure = set_writefunction conn closure
+    method set_writefunction2 closure = set_writefunction2 conn closure
     method set_readfunction closure = set_readfunction conn closure
+    method set_readfunction2 closure = set_readfunction2 conn closure
     method set_infilesize size = set_infilesize conn size
     method set_url url = set_url conn url
     method set_proxy proxy = set_proxy conn proxy
@@ -1249,6 +1354,7 @@ class handle =
     method set_interface interface = set_interface conn interface
     method set_krb4level krb4level = set_krb4level conn krb4level
     method set_progressfunction closure = set_progressfunction conn closure
+    method set_xferinfofunction closure = set_xferinfofunction conn closure
     method set_sslverifypeer flag = set_sslverifypeer conn flag
     method set_cainfo cainfo = set_cainfo conn cainfo
     method set_capath capath = set_capath conn capath
@@ -1289,6 +1395,7 @@ class handle =
     method set_postfieldsize size = set_postfieldsize conn size
     method set_postfieldsizelarge size = set_postfieldsizelarge conn size
     method set_tcpnodelay flag = set_tcpnodelay conn flag
+    method set_tcpfastopen flag = set_tcpfastopen conn flag
     method set_ftpsslauth flag = set_ftpsslauth conn flag
     method set_ioctlfunction closure = set_ioctlfunction conn closure
     method set_ftpaccount account = set_ftpaccount conn account
@@ -1322,10 +1429,14 @@ class handle =
     method set_dns_servers l = set_dns_servers conn l
     method set_autoreferer b = set_autoreferer conn b
     method set_opensocketfunction closure = set_opensocketfunction conn closure
+(*     method set_closesocketfunction closure = set_closesocketfunction conn closure *)
     method set_proxytype t = set_proxytype conn t
     method set_mimepost p = set_mimepost conn p
     method set_sshknownhosts s = set_sshknownhosts conn s
     method set_sshkeyfunction f = set_sshkeyfunction conn f
+    method set_buffersize buffersize = set_buffersize conn buffersize
+    method set_doh_url url = set_doh_url conn url
+    method set_ssl_options opts = set_ssl_options conn opts
 
     method get_effectiveurl = get_effectiveurl conn
     method get_redirecturl = get_redirecturl conn
@@ -1358,12 +1469,14 @@ class handle =
     method get_sslengines = get_sslengines conn
     method get_cookielist = get_cookielist conn
     method get_lastsocket = get_lastsocket conn
+    method get_activesocket = get_activesocket conn
     method get_ftpentrypath = get_ftpentrypath conn
     method get_primaryip = get_primaryip conn
     method get_localip = get_localip conn
     method get_localport = get_localport conn
     method get_conditionunmet = get_conditionunmet conn
     method get_certinfo = get_certinfo conn
+    method get_http_version = get_http_version conn
 end
 
 module Multi = struct
@@ -1377,16 +1490,24 @@ module Multi = struct
     | CURLMOPT_MAXCONNECTS of int
     | CURLMOPT_MAX_PIPELINE_LENGTH of int
     | CURLMOPT_MAX_HOST_CONNECTIONS of int
+    | CURLMOPT_MAX_TOTAL_CONNECTIONS of int
 
   exception Error of string
 
+  type cerror = int
+
+  exception CError of string * cerror * string
+
   let () = Callback.register_exception "Curl.Multi.Error" (Error "")
+  let () = Callback.register_exception "Curl.Multi.CError" (CError ("",0,""))
 
   external create : unit -> mt = "caml_curl_multi_init"
   external add : mt -> t -> unit = "caml_curl_multi_add_handle"
   external perform : mt -> int = "caml_curl_multi_perform_all"
   external wait : int -> mt -> bool = "caml_curl_multi_wait"
   let wait ?(timeout_ms=1000) mt = wait timeout_ms mt
+  external poll : int -> mt -> bool = "caml_curl_multi_poll"
+  let poll ?(timeout_ms=1000) mt = poll timeout_ms mt
   external remove : mt -> t -> unit = "caml_curl_multi_remove_handle"
   external remove_finished : mt -> (t * curlCode) option = "caml_curlm_remove_finished"
   external cleanup : mt -> unit = "caml_curl_multi_cleanup"
