@@ -255,9 +255,18 @@ type curlProxyType =
   | CURLPROXY_SOCKS4A (** since libcurl 7.18.0 *)
   | CURLPROXY_SOCKS5_HOSTNAME (** since libcurl 7.18.0 *)
 
+type data_source =
+  | String of string (** Equivalent to `CURLMIME_DATA` *)
+  | File of string  (** Equivalent to `CURLMIME_FILEDATA` *)
+
 type curlMIMEPartData =
   | CURLMIME_DATA of string
   | CURLMIME_FILEDATA of string
+  | CURLMIME_DATA_WITH_NAME of {
+      data: data_source;
+      name: string option;
+      filename: string option;
+    }
 
 type curlMIMEEncoding =
   | CURLMIME_8BIT
@@ -325,6 +334,21 @@ type curlPostRedir =
 | REDIR_POST_302
 | REDIR_POST_303 (** added in libcurl 7.26.0 *)
 
+type curlSslOption =
+  | CURLSSLOPT_ALLOW_BEAST
+  | CURLSSLOPT_NO_REVOKE
+  | CURLSSLOPT_NO_PARTIALCHAIN
+  | CURLSSLOPT_REVOKE_BEST_EFFORT
+  | CURLSSLOPT_NATIVE_CA
+  | CURLSSLOPT_AUTO_CLIENT_CERT
+
+type 'a xfer_result = Proceed of 'a | Pause | Abort
+
+type write_result = unit xfer_result
+type read_result = string xfer_result
+
+val proceed : write_result
+
 type curlOption =
   | CURLOPT_WRITEFUNCTION of (string -> int)
   | CURLOPT_READFUNCTION of (int -> string)
@@ -422,6 +446,7 @@ type curlOption =
   | CURLOPT_FTPSSL of curlFTPSSL
   | CURLOPT_POSTFIELDSIZELARGE of int64
   | CURLOPT_TCPNODELAY of bool
+  | CURLOPT_TCP_FASTOPEN of bool
   | CURLOPT_FTPSSLAUTH of curlFTPSSLAuth
   | CURLOPT_IOCTLFUNCTION of (t -> curlIOCmd -> curlIOErr)
   | CURLOPT_FTPACCOUNT of string
@@ -453,6 +478,7 @@ type curlOption =
   | CURLOPT_SEEKFUNCTION of (int64 -> curlSeek -> curlSeekResult)
   | CURLOPT_AUTOREFERER of bool
   | CURLOPT_OPENSOCKETFUNCTION of (Unix.file_descr -> unit)
+(*   | CURLOPT_CLOSESOCKETFUNCTION of (Unix.file_descr -> unit) *)
   | CURLOPT_PROXYTYPE of curlProxyType
   | CURLOPT_PROTOCOLS of curlProto list
   | CURLOPT_REDIR_PROTOCOLS of curlProto list
@@ -473,6 +499,12 @@ type curlOption =
   (** The second argument to the passed function consists of the raw bytes of
       the public key sent by the remote host. If the function raises an
       exception the key will be rejected, and the connection will fail.**)
+  | CURLOPT_BUFFERSIZE of int
+  | CURLOPT_DOH_URL of string
+  | CURLOPT_SSL_OPTIONS of curlSslOption list
+  | CURLOPT_WRITEFUNCTION2 of (string -> write_result)
+  | CURLOPT_READFUNCTION2 of (int -> read_result)
+  | CURLOPT_XFERINFOFUNCTION of (int64 -> int64 -> int64 -> int64 -> bool)
 
 type initOption =
   | CURLINIT_GLOBALALL
@@ -518,6 +550,8 @@ type curlInfo =
   | CURLINFO_LOCAL_PORT
   | CURLINFO_CONDITION_UNMET
   | CURLINFO_CERTINFO
+  | CURLINFO_ACTIVESOCKET
+  | CURLINFO_HTTP_VERSION
 
 type curlInfoResult =
   | CURLINFO_String of string
@@ -525,6 +559,27 @@ type curlInfoResult =
   | CURLINFO_Double of float
   | CURLINFO_StringList of string list
   | CURLINFO_StringListList of string list list
+  | CURLINFO_Socket of Unix.file_descr
+  | CURLINFO_Version of curlHTTPVersion
+
+type curlSslBackend =
+  | CURLSSLBACKEND_NONE
+  | CURLSSLBACKEND_OPENSSL
+  | CURLSSLBACKEND_GNUTLS
+  | CURLSSLBACKEND_NSS
+  | CURLSSLBACKEND_GSKIT
+  | CURLSSLBACKEND_WOLFSSL
+  | CURLSSLBACKEND_SCHANNEL
+  | CURLSSLBACKEND_SECURETRANSPORT
+  | CURLSSLBACKEND_MBEDTLS
+  | CURLSSLBACKEND_MESALINK
+  | CURLSSLBACKEND_BEARSSL
+
+type curlSslSet =
+  | CURLSSLSET_OK
+  | CURLSSLSET_UNKNOWN_BACKEND
+  | CURLSSLSET_TOO_LATE
+  | CURLSSLSET_NO_BACKENDS
 
 type version_info = {
   version : string;
@@ -543,6 +598,22 @@ type version_info = {
 
 type pauseOption = PAUSE_SEND | PAUSE_RECV | PAUSE_ALL
 
+(** {2 MultiSSL mode } *)
+
+exception CurlSslSetException of curlSslSet
+
+val global_sslset : curlSslBackend -> unit
+(** @since libcurl 7.56.0 *)
+
+val global_sslset_str : string -> unit
+(** @since libcurl 7.56.0 *)
+
+val global_sslsetavail : unit -> curlSslBackend list
+(** @since libcurl 7.56.0 *)
+
+val global_sslsetavail_str : unit -> string list
+(** @since libcurl 7.56.0 *)
+
 (** {2 curl_easy API} *)
 
 val global_init : initOption -> unit
@@ -565,6 +636,8 @@ val curlCode_of_int : int -> curlCode option
 (** same as [int_of_curlCode] *)
 val errno : curlCode -> int
 val version_info : unit -> version_info
+
+(** flags set the new state, ie to unpause - pass empty list *)
 val pause : t -> pauseOption list -> unit
 
 (** {2 Set transfer options}
@@ -574,9 +647,19 @@ val pause : t -> pauseOption list -> unit
   and transfer will be aborted. *)
 
 val set_writefunction : t -> (string -> int) -> unit
-val set_readfunction : t -> (int -> string) -> unit
+
+(* Alternative API for the write callback that allows to pause download
+   NB to unpause - call [Curl.pause h \[\]] from progressfunction callback (which is called every second at least),
+   do not try to call unpause from another thread, see libcurl documentation for details *)
+val set_writefunction2 : t -> (string -> write_result) -> unit
+
 (** [readfunction n] should return string of length at most [n], otherwise
   transfer will be aborted (as if with exception) *)
+val set_readfunction : t -> (int -> string) -> unit
+
+(* Alternative API for the read callback that allows to pause upload *)
+val set_readfunction2 : t -> (int -> read_result) -> unit
+
 val set_infilesize : t -> int -> unit
 val set_url : t -> string -> unit
 val set_proxy : t -> string -> unit
@@ -633,7 +716,10 @@ val set_timevalue : t -> int32 -> unit
 val set_customrequest : t -> string -> unit
 val set_interface : t -> string -> unit
 val set_krb4level : t -> curlKRB4Level -> unit
+
+(** callback returns whether transfer should be interrupted, ie return false to continue transfering *)
 val set_progressfunction : t -> (float -> float -> float -> float -> bool) -> unit
+val set_xferinfofunction : t -> (int64 -> int64 -> int64 -> int64 -> bool) -> unit
 val set_sslverifypeer : t -> bool -> unit
 val set_cainfo : t -> string -> unit
 val set_capath : t -> string -> unit
@@ -672,6 +758,7 @@ val set_netrcfile : t -> string -> unit
 val set_ftpssl : t -> curlFTPSSL -> unit
 val set_postfieldsizelarge : t -> int64 -> unit
 val set_tcpnodelay : t -> bool -> unit
+val set_tcpfastopen : t -> bool -> unit
 val set_ftpsslauth : t -> curlFTPSSLAuth -> unit
 val set_ioctlfunction : t -> (t -> curlIOCmd -> curlIOErr) -> unit
 val set_ftpaccount : t -> string -> unit
@@ -703,9 +790,17 @@ val set_proxytransfermode : t -> bool -> unit
 val set_seekfunction : t -> (int64 -> curlSeek -> curlSeekResult) -> unit
 val set_autoreferer : t -> bool -> unit
 val set_opensocketfunction : t -> (Unix.file_descr -> unit) -> unit
+
+(** current implementation is faulty
+    ref https://github.com/ygrek/ocurl/issues/58
+val set_closesocketfunction : t -> (Unix.file_descr -> unit) -> unit
+*)
 val set_proxytype : t -> curlProxyType -> unit
 val set_protocols : t -> curlProto list -> unit
 val set_redirprotocols : t -> curlProto list -> unit
+val set_buffersize : t -> int -> unit
+val set_doh_url : t -> string -> unit
+val set_ssl_options : t -> curlSslOption list -> unit
 
 (** [set_resolve t add del] adjusts builtin dns mapping
 
@@ -766,6 +861,7 @@ val get_numconnects : t -> int
 val get_sslengines : t -> string list
 val get_cookielist : t -> string list
 val get_lastsocket : t -> int
+val get_activesocket : t -> Unix.file_descr option
 val get_ftpentrypath : t -> string
 val get_primaryip : t -> string
 val get_localip : t -> string
@@ -775,6 +871,8 @@ val get_localport : t -> int
 val get_conditionunmet : t -> bool
 (** @since 0.6.1 (libcurl 7.19.4) *)
 val get_certinfo : t -> string list list
+(** @since 0.9.2 *)
+val get_http_version : t -> curlHTTPVersion
 
 (** {2 Object interface} *)
 
@@ -786,7 +884,9 @@ class handle :
     method perform : unit
 
     method set_writefunction : (string -> int) -> unit
+    method set_writefunction2 : (string -> write_result) -> unit
     method set_readfunction : (int -> string) -> unit
+    method set_readfunction2 : (int -> read_result) -> unit
     method set_infilesize : int -> unit
     method set_url : string -> unit
     method set_proxy : string -> unit
@@ -845,6 +945,8 @@ class handle :
     method set_krb4level : curlKRB4Level -> unit
     method set_progressfunction :
       (float -> float -> float -> float -> bool) -> unit
+    method set_xferinfofunction :
+      (int64 -> int64 -> int64 -> int64 -> bool) -> unit
     method set_sslverifypeer : bool -> unit
     method set_cainfo : string -> unit
     method set_capath : string -> unit
@@ -883,6 +985,7 @@ class handle :
     method set_ftpssl : curlFTPSSL -> unit
     method set_postfieldsizelarge : int64 -> unit
     method set_tcpnodelay : bool -> unit
+    method set_tcpfastopen : bool -> unit
     method set_ftpsslauth : curlFTPSSLAuth -> unit
     method set_ioctlfunction : (t -> curlIOCmd -> curlIOErr) -> unit
     method set_ftpaccount : string -> unit
@@ -914,12 +1017,16 @@ class handle :
     method set_seekfunction : (int64 -> curlSeek -> curlSeekResult) -> unit
     method set_autoreferer : bool -> unit
     method set_opensocketfunction : (Unix.file_descr -> unit) -> unit
+(*     method set_closesocketfunction : (Unix.file_descr -> unit) -> unit *)
     method set_proxytype : curlProxyType -> unit
     method set_resolve : (string * int * string) list -> (string * int) list -> unit
     method set_dns_servers : string list -> unit
     method set_mimepost : curlMIMEPart list -> unit
     method set_sshknownhosts : string -> unit
     method set_sshkeyfunction : (curlKHMatch -> string -> curlKHStat) -> unit
+    method set_buffersize : int -> unit
+    method set_doh_url : string -> unit
+    method set_ssl_options : curlSslOption list -> unit
 
     method get_effectiveurl : string
     method get_redirecturl : string
@@ -952,12 +1059,14 @@ class handle :
     method get_sslengines : string list
     method get_cookielist : string list
     method get_lastsocket : int
+    method get_activesocket : Unix.file_descr option
     method get_ftpentrypath : string
     method get_primaryip : string
     method get_localip : string
     method get_localport : int
     method get_conditionunmet : bool
     method get_certinfo : string list list
+    method get_http_version : curlHTTPVersion
   end
 
 (** {2 curl_multi API} *)
@@ -975,10 +1084,16 @@ module Multi : sig
     | CURLMOPT_MAXCONNECTS of int
     | CURLMOPT_MAX_PIPELINE_LENGTH of int
     | CURLMOPT_MAX_HOST_CONNECTIONS of int
+    | CURLMOPT_MAX_TOTAL_CONNECTIONS of int
 
   (** exception raised on internal errors *)
   exception Error of string
 
+  type cerror
+
+  (** exception raised on libcurl errors : origin * code * message *)
+  exception CError of string * cerror * string
+
   (** create new multi stack *)
   val create : unit -> mt
 
@@ -996,6 +1111,13 @@ module Multi : sig
       @return whether [perform] should be called *)
   val wait : ?timeout_ms:int -> mt -> bool
 
+  (** poll till there are some active data transfers on multi stack.
+      Contrary to [wait], this function does not return immediately
+      when there are no pending transfer but waits for [timeout_ms]
+      The module falls back to [wait] if this function is unavailable.
+      @return whether [perform] should be called *)
+  val poll : ?timeout_ms:int -> mt -> bool
+
   (** remove finished handle from the multi stack if any. The returned handle may be reused *)
   val remove_finished : mt -> (t * curlCode) option
 
